#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Drawing;

namespace HeuristicLab.Optimization.Networks {
  [Item("Port", "Abstract base class for ports of an optimization network node.")]
  [StorableClass]
  public abstract class Port : NamedItem, IPort {
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Field; }
    }

    [Storable]
    private INode node;
    public INode Node {
      get { return node; }
      set {
        if ((value != null) && !value.Ports.Contains(this))
          throw new InvalidOperationException("Port is not contained in port collection of node.");
        if ((value == null) && node.Ports.Contains(this))
          throw new InvalidOperationException("Port is still contained in port collection of node.");
        node = value;
      }
    }

    [StorableConstructor]
    protected Port(bool deserializing) : base(deserializing) { }
    protected Port(Port original, Cloner cloner)
      : base(original, cloner) {
      node = (INode)cloner.GetClone(original.node);  // NOTE: original node is NOT automatically cloned
    }
    protected Port() : base("Port") { }
    protected Port(string name) : base(name) { }
    protected Port(string name, string description) : base(name, description) { }
  }
}
