#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System;
using System.Collections.Generic;

namespace HeuristicLab.Optimization.Networks {
  [StorableClass]
  [Item("PortCollection", "Represents a collection of ports in an optimization network node.")]
  public class PortCollection : NamedItemCollection<IPort> {
    [Storable]
    private INode node;
    public INode Node {
      get { return node; }
    }

    [StorableConstructor]
    protected PortCollection(bool deserializing) : base(deserializing) { }
    protected PortCollection(PortCollection original, Cloner cloner)
      : base(original, cloner) {
      if (!cloner.ClonedObjectRegistered(original.node))
        // NOTE: cannot clone port collection alone if referenced node is not cloned
        throw new InvalidOperationException("Node of port collection is not cloned.");
      node = cloner.Clone(original.node);
    }
    public PortCollection(INode node)
      : base() {
      this.node = node;
    }
    public PortCollection(INode node, int capacity)
      : base(capacity) {
      this.node = node;
    }
    public PortCollection(INode node, IEnumerable<IPort> collection)
      : base(collection) {
      this.node = node;
    }

    public override IDeepCloneable Clone(Cloner cloner) { return new PortCollection(this, cloner); }

    protected override void OnItemsAdded(IEnumerable<IPort> items) {
      foreach (var p in items)
        p.Node = node;
      base.OnItemsAdded(items);
    }
    protected override void OnItemsRemoved(IEnumerable<IPort> items) {
      foreach (var p in items)
        p.Node = null;
      base.OnItemsRemoved(items);
    }
    protected override void OnItemsReplaced(IEnumerable<IPort> items, IEnumerable<IPort> oldItems) {
      foreach (var p in oldItems)
        p.Node = null;
      foreach (var p in items)
        p.Node = node;
      base.OnItemsReplaced(items, oldItems);
    }
    protected override void OnCollectionReset(IEnumerable<IPort> items, IEnumerable<IPort> oldItems) {
      foreach (var p in oldItems)
        p.Node = null;
      foreach (var p in items)
        p.Node = node;
      base.OnCollectionReset(items, oldItems);
    }
  }
}
